/** 
 * XMLSec library
 *
 * Keys Base: forward declarations
 *
 * See Copyright for the status of this software.
 * 
 * Author: Aleksey Sanin <aleksey@aleksey.com>
 */
#ifndef __XMLSEC_KEYS_H__
#define __XMLSEC_KEYS_H__    

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */ 


#include <xmlsec/xmlsec.h>
#ifndef XMLSEC_NO_X509
#include <openssl/x509.h>
#endif /* XMLSEC_NO_X509 */

#include <xmlsec/xmlsec.h>

/** 
 * Key Types
 */
typedef enum _xmlSecKeyType {
    xmlSecKeyTypePublic = 0,
    xmlSecKeyTypePrivate,
    xmlSecKeyTypeAny
} xmlSecKeyType;

/** 
 * Key Origins
 */
typedef long				xmlSecKeyOrigin;
#define xmlSecKeyOriginDefault			0
#define xmlSecKeyOriginKeyManager		1
#define xmlSecKeyOriginKeyName			2 /* useless w/o xmlSecKeyOriginKeyManager */
#define xmlSecKeyOriginKeyValue			4
#define xmlSecKeyOriginRetrievalDocument	8
#define xmlSecKeyOriginRetrievalRemote		16
#define xmlSecKeyOriginX509			32
#define xmlSecKeyOriginPGP			64
#define xmlSecKeyOriginEncryptedKey		128
#define xmlSecKeyOriginAll			\
	    (xmlSecKeyOriginKeyManager | xmlSecKeyOriginKeyName | \
	     xmlSecKeyOriginKeyValue | xmlSecKeyOriginKeyValue | \
	     xmlSecKeyOriginRetrievalDocument | xmlSecKeyOriginRetrievalRemote | \
	     xmlSecKeyOriginX509 | xmlSecKeyOriginPGP | xmlSecKeyOriginEncryptedKey)		


typedef struct _xmlSecKey 		*xmlSecKeyPtr; 
typedef const struct _xmlSecKeyId	*xmlSecKeyId; 

#define xmlSecKeyIdUnknown 			NULL


#include <xmlsec/x509.h>

struct _xmlSecKey {
    xmlSecKeyId				id;
    xmlSecKeyType			type;
    xmlChar				*name;
    xmlSecKeyOrigin			origin;

#ifndef XMLSEC_NO_X509    
    /* x509 cert used to create the key (if any) */
    X509				*x509Cert;    
    /* x509 certs that verifies the key (including the x509Cert!) */
    STACK_OF(X509) 			*x509Chain;
#endif /* XMLSEC_NO_X509 */
    
    /* key specific data */
    void				*keyData;
};



typedef xmlSecKeyPtr 	(*xmlSecKeyFindKeyCallback)	(void *context,
							 xmlSecKeyId id, 
							 xmlSecKeyType type,
						         const xmlChar *name);
typedef struct _xmlSecKeysReadContext {
    xmlSecKeyOrigin 			allowedOrigins;
    int 				maxRetrievals;
    int					maxEncKeyLevels;
    
    /* key manager context */
    xmlSecKeyFindKeyCallback 		findKeyCallback;
    void 				*findKeyContext;   

#ifndef XMLSEC_NO_X509
    xmlSecX509Context			x509;
#endif /* XMLSEC_NO_X509 */    
} xmlSecKeysReadContext;


xmlSecKeyPtr	xmlSecKeyCreate				(xmlSecKeyId id,
							 xmlSecKeyOrigin origin);
void		xmlSecKeyDestroy			(xmlSecKeyPtr key,
							 int forceDestroy);
void		xmlSecDebugKeyDump			(FILE *output,
							 xmlSecKeyPtr key);


#include <xmlsec/transforms.h>
xmlNodePtr	xmlKeyInfoAddKeyName			(xmlNodePtr keyInfoNode);
xmlNodePtr	xmlKeyInfoAddKeyValue			(xmlNodePtr keyInfoNode);
xmlNodePtr	xmlKeyInfoAddX509Data			(xmlNodePtr keyInfoNode);
xmlNodePtr	xmlKeyInfoAddRetrievalMethod		(xmlNodePtr keyInfoNode,
							 const xmlChar *uri,
							 const xmlChar *type);
xmlNodePtr	xmlKeyInfoAddRetrievalMethodTransform	(xmlNodePtr keyInfoNode,
							 xmlSecTransformId id);
							 							 
    
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __XMLSEC_KEYS_H__ */

