/** 
 * XMLSec library
 *
 * Keys
 *
 * See Copyright for the status of this software.
 * 
 * Author: Aleksey Sanin <aleksey@aleksey.com>
 */
#ifndef __XMLSEC_KEYS_INTERNAL_H__
#define __XMLSEC_KEYS_INTERNAL_H__    

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */ 

#include <libxml/tree.h>
#include <openssl/evp.h>

#ifndef XMLSEC_NO_X509
#include <openssl/x509.h>
#endif /* XMLSEC_NO_X509 */

#include <xmlsec/xmlsec.h>
#include <xmlsec/keys.h>
#include <xmlsec/transforms.h>
#include <xmlsec/x509.h>

/**
 * inifinite number of retrievals (really big number :) )
 */
#define xmlSecKeyInifiteRetrivals		99999

typedef struct _xmlSecKeysReadContext	*xmlSecKeysReadContextPtr;
    
typedef xmlSecKeyPtr	(*xmlSecKeyCreateMethod)	(xmlSecKeyId id);
typedef void		(*xmlSecKeyDestroyMethod)	(xmlSecKeyPtr key);
typedef int		(*xmlSecKeyReadXmlMethod)	(xmlSecKeyPtr key,
							 xmlNodePtr node);
typedef int		(*xmlSecKeyWriteXmlMethod)	(xmlSecKeyPtr key,
							 xmlSecKeyType type,
							 xmlNodePtr parent);
typedef int		(*xmlSecKeyReadBinaryMethod)	(xmlSecKeyPtr key,
							 const unsigned char *buf,
							 size_t size);
typedef int		(*xmlSecKeyWriteBinaryMethod)	(xmlSecKeyPtr key,
							 xmlSecKeyType type,
							 unsigned char **buf,
							 size_t *size);


struct _xmlSecKeyId {
    /* xlmlSecKeyId data */
    const xmlChar 			*keyValueNodeName;
    const xmlChar			*keyValueNodeNs;
    
    /* xmlSecKeyId methods */
    xmlSecKeyCreateMethod		create;
    xmlSecKeyDestroyMethod		destroy;
    xmlSecKeyReadXmlMethod		read;
    xmlSecKeyWriteXmlMethod		write;
    xmlSecKeyReadBinaryMethod		readBin;
    xmlSecKeyWriteBinaryMethod		writeBin;
};


/** 
 * XML Sec Key
 */
void		xmlSecKeysInit				(void); 
 
 
#define xmlSecKeyIsValid(key) \
	((( key ) != NULL) && ((( key )->id) != NULL))
#define xmlSecKeyCheckId(key, keyId) \
 	(xmlSecKeyIsValid(( key )) && \
	((( key )->id) == ( keyId )))
#define xmlSecKeyCheckTransform(key, tr) \
 	(xmlSecKeyIsValid(( key )) && \
	((((const xmlSecKeyId) (( key )->id->transformId))) == ( tr )))
xmlSecKeyPtr	xmlSecKeyReadEvpKey			(EVP_PKEY *pKey, 
							 xmlSecTransformId id, 
							 xmlSecKeyOrigin origin);
xmlSecKeyPtr	xmlSecKeyReadXml			(xmlSecKeyId id,
							 xmlNodePtr node);
int		xmlSecKeyWriteXml			(xmlSecKeyPtr key,
							 xmlSecKeyType type,
							 xmlNodePtr node); 							
xmlSecKeyPtr	xmlSecKeyReadBin			(xmlSecKeyId id,
							 const unsigned char *buf,
							 size_t size);
int		xmlSecKeyWriteBin			(xmlSecKeyPtr key,
							 xmlSecKeyType type,
							 unsigned char **buf,
							 size_t *size);
/** 
 * XMLDsig:KeyInfo methods
 */ 
xmlSecKeyPtr	xmlSecKeyInfoRead			(const xmlNodePtr keyInfoNode,
							 xmlSecTransformId id,
							 xmlSecKeyType type,
							 const xmlSecKeysReadContext * ctx);
int		xmlSecKeyInfoWrite			(xmlNodePtr keyInfoNode,
							 const xmlSecKeyPtr key,
							 xmlSecKeyType type,
							 const xmlSecKeysReadContext *ctx);

/** 
 *
 */							 
xmlSecKeyPtr 	xmlSecKeySearcCtxFindKey		(const xmlSecKeysReadContext *keysContext,
							 xmlSecTransformId id,
							 xmlSecKeyType type,
							 const xmlChar *name);
    
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __XMLSEC_KEYS_INTERNAL_H__ */

