/** 
 * XMLSec library
 *
 * X509 support
 *
 * See Copyright for the status of this software.
 * 
 * Author: Aleksey Sanin <aleksey@aleksey.com>
 */
#ifndef __XMLSEC_X509_H__
#define __XMLSEC_X509_H__    

#ifndef XMLSEC_NO_X509

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */ 

#include <libxml/tree.h>

#include <openssl/x509.h>

#include <xmlsec/xmlsec.h>
#include <xmlsec/keys.h>
#include <xmlsec/transforms.h>

/**
 * xmlSecX509FindCallback:
 * @context: the application specific context
 * @subjectName: subject name string
 * @issuerName: issuer name string
 * @issuerSerial: issuer serial
 * @ski: ski
 *
 * Returns the certificate that matches given criteria
 */
typedef X509*	(*xmlSecX509FindCallback)		(void *context,
							 xmlChar *subjectName,
							 xmlChar *issuerName,
							 xmlChar *issuerSerial,
							 xmlChar *ski);
/**
 * xmlSecX509VerifyCallback:
 * @context: the application specific context
 * @cert: the cert to verify
 * @untrusted: untrusted ceritficates from the chain
 *
 * Returns 1 if the cert is trusted, 0 if it is not trusted
 * and -1 if an error occurs
 */
typedef int	(*xmlSecX509VerifyCallback)		(void *context,
							 X509 *cert,
						         STACK_OF(X509) *untrusted);
/** 
 * xmlSecX509AddCRLCallback:
 * @context:
 * @crl:	the verified CRL to add
 *
 */
typedef int	(*xmlSecX509AddCRLCallback)		(void *context,
							 X509_CRL *crl);

typedef struct _xmlSecX509Context {
    /* x509 context */
    xmlSecX509FindCallback	findCallback;
    xmlSecX509VerifyCallback	verifyCallback;
    xmlSecX509AddCRLCallback	addCRLCallback;
    void			*context;
} xmlSecX509Context, *xmlSecX509ContextPtr;

xmlSecKeyPtr	xmlSecX509DataNodeRead			(const xmlNodePtr x509DataNode,
							 xmlSecTransformId id,
							 xmlSecKeyType type,
							 const xmlSecX509Context *ctx); 
int		xmlSecX509DataNodeWrite			(xmlNodePtr x509DataNode,
							 const xmlSecKeyPtr key);

void		xmlSecDebugx509Dump			(FILE *output, 
							 X509 *cert, 
							 STACK_OF(X509) *chain);

xmlSecKeyPtr	xmlSecDerX509Read			(unsigned char *buf,
							 size_t size,
							 xmlSecTransformId id,
							 xmlSecKeyType type,
							 const xmlSecX509Context *ctx); 

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* XMLSEC_NO_X509 */

#endif /* __XMLSEC_X509_H__ */

